@@ -25,6 +25,7 @@
 #import <Foundation/NSObject.h>
 #import <Foundation/NSString.h>
 #include <NGStreams/NGStreamProtocols.h>
+#import <Foundation/NSFileHandle.h>
 
 /*
   WOMessage
@@ -41,6 +42,7 @@
 @private
   NSString         *version;
   NSMutableData    *content;
+  NSFileHandle     *contentFileHandle;
   NGMutableHashMap *header;
   NSDictionary     *userInfo;
   NSMutableArray   *cookies;
@@ -102,6 +104,11 @@
 + (void)setDefaultEncoding:(NSStringEncoding)_encoding;
 + (NSStringEncoding)defaultEncoding;
 
+/* file content */
+
+- (void)setContentFile:(NSFileHandle *) handle;
+- (NSFileHandle *)contentFile;
+
 /* structured content */
 
 - (void)appendContentBytes:(const void *)_bytes length:(unsigned)_length;
