@@ -49,7 +49,6 @@
   NGImap4Client *client;
   NSString      *password;
   NSDate        *creationTime;
-  NSString      *separator;
 
   /* hierarchy cache */
   NSMutableDictionary  *subfolders;
@@ -102,6 +101,8 @@
 - (NSArray *)subfoldersForURL:(NSURL *)_url
   onlySubscribedFolders: (BOOL) subscribedFoldersOnly;
 - (NSArray *)allFoldersForURL:(NSURL *)_url;
+- (NSDictionary *) allFoldersMetadataForURL:(NSURL *)_url
+                      onlySubscribedFolders:(BOOL)_subscribedFoldersOnly;
 - (NSArray *)allFoldersForURL:(NSURL *)_url
   onlySubscribedFolders: (BOOL) subscribedFoldersOnly;
 - (BOOL)selectFolder:(id)_url;
