@@ -25,15 +25,6 @@
 
 @implementation NGMimeMessageBodyGenerator
 
-+ (int)version {
-  return 2;
-}
-+ (void)initialize {
-  NSAssert2([super version] == 2,
-            @"invalid superclass (%@) version %i !",
-            NSStringFromClass([self superclass]), [super version]);
-}
-
 /* encoding data */
 
 - (NSData *)encodeData:(NSData *)_data
